#include <PalmOS.h>
#include "resource.h"
#include "mymath.h"
#include "physics.h"
#include "ele101.h"

#define GAME_CYCLE_TIME       10       /* 0.10 seconds */

RectangleType displayrect;
BitmapType* backgroundbitmapP;
MemHandle backgroundbitmapH;
WinHandle OffscreenH;

ball_t clearball = { { 40, 80 }, { 75, -5 } };
ball_t darkball = { { 100, 80 }, { 0, 0 } };

void DrawDarkBall(Int16 x, Int16 y) {
    RectangleType rect;
    rect.topLeft.x = x - BALL_RADIUS;
    rect.topLeft.y = y - BALL_RADIUS;
    rect.extent.x = 2 * BALL_RADIUS;
    rect.extent.y = 2 * BALL_RADIUS;
    WinDrawRectangle(&rect, BALL_RADIUS);
}

void WinDrawBitmapTransparent(BitmapPtr bitmapP, Coord x, Coord y) {
    /* Draws only the black pixels of a monochrome (black/white) bitmap */
    Char* bits, *ptr;
    UInt8 mask;
    Int16 i, j;
    ErrFatalDisplayIf(backgroundbitmapP->pixelSize != 1,
      "Transparent drawing supported for only monochrome bitmaps");
    bits = (Char*) BmpGetBits(bitmapP);
    for (i = 0; i < bitmapP->height; i += 1) {
        ptr = bits + bitmapP->rowBytes * i;
        mask = 0x80;
        for (j = 0; j < bitmapP->width; j += 1) {
            if (*ptr & mask)
                WinDrawPixel(x + j, y + i);
            mask = mask >> 1;
            if (mask == 0) {
                mask = 0x80;
                ptr += 1;
            }
        }
    }
}

void game_cycle() {
    /* This function called routinely after the appropriate number of
     * ticks have expired. */
    if (DetectCollision(clearball, darkball) ||
        DetectCollision(darkball, clearball)) {
        vector_t from_clearball, from_darkball;
        from_clearball = Collide(clearball, darkball);
        from_darkball  = Collide(darkball, clearball);
        darkball.velocity  = AddVectors(darkball.velocity, from_clearball);
        clearball.velocity = AddVectors(clearball.velocity, NegateVector(from_clearball));
        clearball.velocity = AddVectors(clearball.velocity, from_darkball);
        darkball.velocity  = AddVectors(darkball.velocity, NegateVector(from_darkball));
    }
    clearball = BounceWall(clearball);
    darkball  = BounceWall(darkball);
    clearball = MoveAndDecelerate(clearball, GAME_CYCLE_TIME / 100.0);
    darkball  = MoveAndDecelerate(darkball, GAME_CYCLE_TIME / 100.0);

    WinDrawBitmap(backgroundbitmapP, displayrect.topLeft.x, displayrect.topLeft.y);
    DrawDarkBall((Int16) clearball.position.x, (Int16) clearball.position.y);
    DrawDarkBall((Int16) darkball.position.x,  (Int16) darkball.position.y);
}

Boolean MainFormHandleEvent(EventPtr evt) {
    Boolean handled = false;
    FormPtr currentform;

    switch (evt->eType) {
        /* frmOpenEvent: called when the form is first opened. */
	  case frmOpenEvent: {
            UInt16 error;
            BitmapPtr formbitmap;

            backgroundbitmapH = DmGetResource('Tbmp', tableimage);
            backgroundbitmapP = (BitmapType*) MemHandleLock(backgroundbitmapH);
            formbitmap = FrmGetObjectPtr(FrmGetActiveForm(),
              FrmGetObjectIndex(FrmGetActiveForm(), tableimage));
            FrmGetObjectBounds(FrmGetActiveForm(), FrmGetObjectIndex(FrmGetActiveForm(),
              tableimage), &displayrect);

            OffscreenH = WinCreateOffscreenWindow(WINDOW_WIDTH, WINDOW_HEIGHT,                        
              screenFormat, &error);
            ErrFatalDisplayIf(error, "Could not create offscreen window");
            currentform = FrmGetActiveForm();
            FrmDrawForm(currentform);
            handled = true;
            break;
        }
        /* frmCloseEvent: Called as we exit the application. */
        case frmCloseEvent:
            MemHandleUnlock(backgroundbitmapH);
            DmReleaseResource(backgroundbitmapH);
            WinDeleteWindow(OffscreenH, false);
            handled = true;
            break;
        /* ctlSelectEvent: A control event has occurred. */
        case ctlSelectEvent:
            /* TODO: Handle events such as the 'New game' button. */
            
            break;
        /* penDownEvent: The screen has been tapped with the stylus. */
        case penDownEvent:
            /* TODO: Handle pen input from the user here. */

            break;
        /* penMoveEvent: A stylus is moved across the screen. */
        case penMoveEvent:
            /* TODO: Handle pen input from the user here. */
 
            break;
        /* nilEvent: We receive this as often as we have requested by SetGameCycle(). */
        case nilEvent:
            game_cycle();
            handled = true;
            break;
        default:
            break;
    }

    return handled;
}

/* Main entry point */

UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    if(cmd == sysAppLaunchCmdNormalLaunch) {
        UInt16 errorcode;
        errorcode = StartApplication();
        if (errorcode != 0)
            return errorcode;
        SetGameCycle(GAME_CYCLE_TIME);
        EventLoop();
        StopApplication();
        return 0;
    } 
    else
        return sysErrParamErr;
}
