#include "mymath.h"

float MySqrt(float a) {
    /* Compute the square root of a number using Newton's method */
    float prev, x;
    if (a == 0.0)
        return 0.0;
    x = a / 2; /* guess */
    do {
        prev = x;
        x = (x + a/x) / 2;
    }
    while (prev - x > 0.0001 || x - prev > 0.00001);
    return x;
}

float MyCos(float a) {
    /* Compute the cosine of an angle in radians using the Taylor series */
    float term = 1.0, total = 1.0;
    int i;
    a -= 2 * PI * (int) (a / 2 / PI);
    if (a < 0)
        a = -a;
    if (a > PI)
        a = 2 * PI - a;
    for (i = 0; i <= 10; i += 2) {
        term = -term * a * a / (i+1) / (i+2);
        total += term;
    }
    return total;
}

float MySin(float a) {
    /* Compute the sine of an angle in radians using the Taylor series */
    float term, total;
    int i;
    a -= 2 * PI * (int) (a / 2 / PI);
    if (a < -PI/2)
        a = -PI - a;
    else if (a > PI/2)
        a = PI - a;
    total = term = a;
    for (i = 1; i <= 11; i += 2) {
        term = -term * a * a / (i+1) / (i+2);
        total += term;
    }
    return total;
}

float MyTan(float a) {
    return MySin(a) / MyCos(a);
}


float MyArcSin(float a) {
    if (a == 1)
        return PI/2;
    else if (a == -1)
        return -PI/2;
    else
        return MyArcTan(a / MySqrt(1 - a*a));
}

float MyArcCos(float a) {
    if (a == 0.0)
        return PI/2;
    else if (a == -0.0)
        return -PI/2;
    else
        return MyArcTan(MySqrt(1 - a*a) / a);
}
 
float MyArcTan(float a) {
    /* Calculate the arctangent using the algorithm from YACAS */
    if (a == 1 / 0.0)
        return PI/2;
    else if (a == -1 / 0.0)
        return -PI/2;
    else if (a < 0)
        return -MyArcTan(-a);
    else if (a > 0.42) {
        float cand1, cand2;
        cand1 = 1/a;
        cand2 = a / (1 + MySqrt(1 + a*a));
        if (cand1 < cand2)
            return PI/2 * MyArcTan(cand1);
        else
            return 2*MyArcTan(cand2);
    }
    else {
        int i;
        float power, total = 0.0;
        power = a;
        for (i = 1; i <= 1001; i += 2) {
            total += power / i;
            power *= -a * a;
        }
        return total;
    }
}

float MagnitudeVector(vector_t v) {
    return MySqrt(v.x * v.x + v.y * v.y);
}

float DotProductVectors(vector_t a, vector_t b) {
    return a.x * b.x + a.y * b.y;
}

vector_t ScalarMultiplyVector(float a, vector_t v) {
    v.x *= a;
    v.y *= a;
    return v;
}

vector_t AddVectors(vector_t a, vector_t b) {
    a.x += b.x;
    a.y += b.y;
    return a;
}

vector_t SubtractVectors(vector_t a, vector_t b) {
    a.x -= b.x;
    a.y -= b.y;
    return a;
}

vector_t NegateVector(vector_t v) {
    v.x = -v.x;
    v.y = -v.y;
    return v;
}

vector_t ProjectVector(vector_t main, vector_t direction) {
    /* Return the projection of vector 'main' onto the vector 'direction' */
    float dot_product, magsquared;
    dot_product = DotProductVectors(main, direction);
    magsquared = DotProductVectors(direction, direction);
    return ScalarMultiplyVector(dot_product / magsquared, direction);
}
